<?php
/**
 * WXR importer class
 *
 * Cron-based importer specifically for WordPress-generated WXR imports.
 *
 * Based on the WordPress Importer
 * (http://wordpress.org/extend/plugins/wordpress-importer/).
 *
 * @package WordPress
 * @subpackage Importer
 */

require_once ABSPATH . '/wp-admin/includes/admin.php';

class WordPress_Importer extends WP_Importer_Cron {

	/**
	 * Highest supported WXR version.
	 *
	 * @var float
	 */
	const MAX_WXR_VERSION = 1.2;

	/**
	 * WXR version of the file to be imported.
	 *
	 * @var float
	 */
	protected $version;

	/**
	 * A single error object representing the fatal problem that halted things.
	 *
	 * @var WP_Error
	 */
	protected $error = null;

	/**
	 * An array of objects representing non-fatal problems that occurred
	 * during import.
	 *
	 * @var WP_Error[]
	 */
	protected $warnings = array();

	/**
	 * Tracks the stage of import.
	 *
	 * @var array
	 */
	protected $progress = array();

	/**
	 * The name of the WXR file being processed by this instance of the importer.
	 *
	 * @var string
	 */
	protected $wxr_file;

	/**
	 * The raw parsed data from the WXR file.
	 *
	 * @var array
	 */
	protected $wxr_data = array();

	/*
	 * SECTION: parsed data from import file
	 */

	/**
	 * Array of user information, including (optionally) names and emails.
	 *
	 * @see process_authors()
	 * @var array
	 */
	protected $authors = array();

	/**
	 * Categories to import.
	 *
	 * @see process_categories()
	 * @var array
	 */
	protected $categories = array();

	/**
	 * Tags to import.
	 *
	 * @see process_tags()
	 * @var array
	 */
	protected $tags = array();

	/**
	 * Other terms (including those of custom taxonomies) to import.
	 *
	 * @see process_terms()
	 * @var array
	 */
	protected $terms = array();

	/**
	 * Posts, attachments, menu items, and custom post type objects to import.
	 *
	 * @see process_posts()
	 * @var array
	 */
	protected $posts = array();

	/*
	 * SECTION: mappings from old information to new
	 */

	/**
	 *
	 * @see process_author_mappings()
	 * @var array
	 */
	protected $processed_authors = array();

	/**
	 *
	 * @see process_author_mappings()
	 * @var array
	 */
	protected $processed_author_mappings = array();

	/**
	 *
	 * @var array
	 */
	protected $processed_terms = array();

	/**
	 *
	 * @var array
	 */
	protected $processed_posts = array();

	/**
	 *
	 * @see process_posts()
	 * @var array
	 */
	protected $processed_post_orphans = array();

	/**
	 *
	 * @see process_posts()
	 * @var array
	 */
	protected $processed_featured_images = array();

	/**
	 *
	 * @see process_menu_item()
	 * @var array
	 */
	protected $processed_menu_items = array();

	/**
	 *
	 * @see process_menu_item()
	 * @var array
	 */
	protected $processed_menu_missing_items = array();

	/**
	 *
	 * @see process_parents()
	 * @var array
	 */
	protected $processed_menu_orphan_items = array();

	/**
	 *
	 * @see fetch_remote_file()
	 * @var array
	 */
	protected $processed_url_remap = array();

	/**
	 * Stores user-submitted choices on how to proceed with the process.
	 *
	 * The first dimension of this array is
	 *
	 * @var array
	 */
	protected $params = array();

	/**
	 * Parses the WXR file and prepares us to process parsed data.
	 *
	 * This method is synchronous and is NOT for use in a cron-scheduled job; it
	 * should be invoked by a controller UI before do_import() is scheduled.
	 *
	 * @param string $file
	 *        	Path to the WXR file for parsing
	 * @param boolean $independent
	 * 			Set to true if the parsing is a trial run or needs to be
	 *          independent of the cron import process; false by default.
	 * @return array WP_Error data if successful, or an instance of WP_Error if
	 *         a problem occurred
	 */
	public function do_parse( $file, $independent = false ) {
		if ( ! is_file( $file ) ) {
			return new WP_Error( 'import_wxr_error',
					__( 'The file does not exist, please try again.',
							'wordpress-importer' ), $file );
		}

		$parser = new WXR_Parser();

		$data = $parser->parse( $file );

		// pass through errors
		if ( is_wp_error( $data ) )
			return $data;

		// check for acceptable version
		$version = floatval( $data['version'] );
		if ( $version > self::MAX_WXR_VERSION ) {
			return new WP_Error( 'import_wxr_version_error',
					__( 'WXR version newer than importer',
							'wordpress-importer' ), $data['version'] );
		}

		$this->wxr_file = $file;
		$this->wxr_data = $data;

		if ( ! $independent ) {
			$this->version = $version;
			$this->categories = $data['categories'];
			$this->tags = $data['tags'];
			$this->terms = $data['terms'];
			$this->posts = $data['posts'];

			if ( ($user = get_current_user_id()) > 0 ) {
				$this->set_param( 'admin_user_id', $user );
			}

			$this->set_param( 'base_url', esc_url( $data['base_url'] ) );

			wp_suspend_cache_invalidation( true );
			wp_suspend_cache_addition( true );

			wp_defer_term_counting( true );
			wp_defer_comment_counting( true );
		}

		/**
		 *
		 * @todo persistent storage without taking up too much space, or more
		 *       efficient parsing of only the section needed
		 */
		return $data;
	}

	/**
	 * The main controller for the actual import stage. Should be called from a
	 * scheduled cron job using
	 * <code>$this->schedule_import_job( 'do_import', array($file) );</code>
	 * but it can be executed directly.
	 *
	 * Modelled on Tumblr_Import::do_blog_import().
	 *
	 * @param string $file
	 *        	Path to the WXR file for importing
	 * @return boolean Whether the import process is complete
	 */
	public function do_import( $file = '' ) {
		// defaults to being done
		$done = true;

		if ( empty( $file ) )
			$file = $this->wxr_file;

		if ( ! empty( $this->progress[$file] ) ) {
			$done = false;

			wp_defer_term_counting( true );
			wp_defer_comment_counting( true );

			do {
				// we could probably be doing call_user_func() or using some
				// Reflection, but this makes more sense for readability and
				// process-specific modifications
				switch ( $this->progress[$file] ) {
					case 'start':
					case 'authors':
						$this->process_authors();
						break;
					case 'author_mappings':
						$this->process_author_mappings();
						break;
					case 'categories':
						$this->process_categories();
						break;
					case 'tags':
						$this->process_tags();
						break;
					case 'terms':
						$this->process_terms();
						break;
					case 'posts':
						$this->process_posts();
						break;
					case 'parents':
						$this->process_parents();
						break;
					case 'urls':
						$this->process_urls();
						break;
					case 'featured_images':
						$this->process_featured_images();
						break;
					case 'finish':
					default:
						$done = true;
				}
				$this->save_vars();
			} while ( empty( $this->error ) && ! $done && $this->have_time() );
		}
		return $done;
	}

	/**
	 * Performs post-import cleanup of files and the cache.
	 *
	 * Must be called to reenable the cache and remove extraneous options.
	 *
	 * If the IMPORT_DEBUG constant is defined and true, the option in the
	 * database will be retained so that the internal data from this class can
	 * be debugged.
	 */
	public function do_cleanup() {
		// cleanup related to the uploaded WXR file should be handled by the
		// external controller, not this model

		wp_cache_flush();
		foreach ( get_taxonomies() as $tax ) {
			delete_option( "{$tax}_children" );
			_get_term_hierarchy( $tax );
		}

		wp_defer_term_counting( false );
		wp_defer_comment_counting( false );

		// un-suspend cache
		wp_suspend_cache_invalidation( false );
		wp_suspend_cache_addition( false );

		if ( ! IMPORT_DEBUG )
			delete_option( get_class( $this ) );
		// do_action('import_end') omitted because that action seems to be
		// designed for callbacks that could sent output
	}

	/**
	 * Advances the progress state to the specified state.
	 *
	 * @param string $step
	 * @return boolean|WP_Error
	 */
	public function advance( $step = '' ) {
		$steps = array(
				'start',
				'authors',
				'author_mappings',
				'categories',
				'tags',
				'terms',
				'posts',
				'parents',
				'urls',
				'featured_images',
				'finish'
		);

		// if no step provided, advance to the next step
		if ( empty( $step ) && ! empty( $this->wxr_file ) ) {
			if ( empty( $this->progress[$this->wxr_file] ) ) {
				// process hasn't yet started; start it
				$step = $steps[0];
			} else {
				$step_index = array_search( $this->progress[$this->wxr_file], $steps );

				// if currently on start step, fast forward 2 steps in total
				if ( $step_index === 0 ) {
					$step_index ++;
				}
				// autoadvance to finish
				if ( $step_index < count( $steps ) )
					$step = $steps[$step_index + 1];
			}
		}
		if ( ! in_array( $step, $steps ) ) {
			return new WP_Error( 'import_cron_step_error',
					__( 'Invalid step', 'wordpress-importer' ) );
		}
		$this->progress[$this->wxr_file] = $step;

		return $step;
	}

	/**
	 * Copies authors data from the WXR parsed data.
	 *
	 * do_parse() must first be called successfully before this can be used.
	 *
	 * Like all process_* methods, this will return true when its step has
	 * completed.
	 *
	 * @param boolean $autoadvance
	 *        	If true (default), advance to the next step
	 *        	automatically after completion; otherwise, do not advance
	 * @return boolean True upon completion, false otherwise, if an error has
	 *         occurred
	 */
	protected function process_authors( $autoadvance = true ) {
		// check for availability of data
		if ( empty( $this->wxr_file ) || empty( $this->wxr_data ) ) {
			$this->error = new WP_Error( 'import_process_wxr_error',
					__( 'No parsed data available for processing',
							'wordpress-importer' ) );
			return false;
		}

		if ( ! empty( $this->wxr_data['authors'] ) ) {
			$this->authors = $this->wxr_data['authors'];
		} else {
			// no author information, grab it from the posts
			foreach ( $this->_['posts'] as $post ) {
				$login = sanitize_user( $post['post_author'], true );
				if ( empty( $login ) ) {
					$this->warnings[] = new WP_Error(
							'import_process_authors_warning',
							sprintf(
									__(
											'Failed to import author %s. Their posts will be attributed to the current user.',
											'wordpress-importer' ),
									esc_html( $post['post_author'] ) ) );
					continue;
				}

				if ( ! isset( $this->authors[$login] ) )
					$this->authors[$login] = array(
							'author_login' => $login,
							'author_display_name' => $post['post_author']
					);
			}
		}

		if ( $autoadvance ) {
			unset($this->wxr_data);
			// done without fatal errors, so we'll advance step
			$this->advance();
		}
		return true;
	}

	/**
	 * Maps old author logins to local user IDs based on decisions.
	 *
	 * Can map to an existing user, create a new user, or falls back to the
	 * current user in case of an error with the other cases.
	 */
	protected function process_author_mappings () {
		$create_users = $this->is_allowed_user_creation();

		// adapted from WP_Import; however, we don't have the luxury of
		// directly accessing $_POST, so we rely instead on DB-stored params
		$imported_authors = $this->get_param( 'imported_authors', array() );

		if ( is_wp_error( $imported_authors ) ) {
			$this->error = $imported_authors;
			return false;
		}

		foreach ( (array) $imported_authors as $i => $old_login ) {
			// Multisite adds strtolower to sanitize_user. Need to sanitize here
			// to stop breakage in process_posts.
			$sanitized_old_login = sanitize_user( $old_login, true );

			/*
			 * $this->authors should have already been set by process_authors(),
			 * although author_id will be missing if author information was
			 * pulled out of post meta
			 */
			$old_id = isset( $this->authors[$old_login]['author_id'] ) ? intval(
					$this->authors[$old_login]['author_id'] ) : false;

			$user_map = $this->get_param( 'user_map', array() );
			if ( ! empty( $user_map[$i] ) ) {
				// the mapping old->new was defined from the form
				$user = get_userdata( intval( $user_map[$i] ) );
				if ( isset( $user->ID ) ) {
					if ( $old_id )
						$this->processed_authors[$old_id] = $user->ID;
					$this->processed_author_mappings[$sanitized_old_login] = $user->ID;
				}
			} elseif ( $create_users ) {
				// make a user for an unmapped author only if allowed
				$user_new = $this->get_param( 'user_new', array() );
				if ( ! empty( $user_new[$i] ) ) {
					$user_id = wp_create_user( $user_new[$i],
							wp_generate_password() );
				} elseif ( $this->version != '1.0' ) {
					$user_data = array(
							'user_login' => $old_login,
							'user_pass' => wp_generate_password(),
							'user_email' => isset(
									$this->authors[$old_login]['author_email'] ) ? $this->authors[$old_login]['author_email'] : '',
							'display_name' => $this->authors[$old_login]['author_display_name'],
							'first_name' => isset(
									$this->authors[$old_login]['author_first_name'] ) ? $this->authors[$old_login]['author_first_name'] : '',
							'last_name' => isset(
									$this->authors[$old_login]['author_last_name'] ) ? $this->authors[$old_login]['author_last_name'] : ''
					);
					$user_id = wp_insert_user( $user_data );
				}

				if ( ! is_wp_error( $user_id ) ) {
					// successful user creation; add to mappings
					if ( $old_id )
						$this->processed_authors[$old_id] = $user_id;
					$this->processed_author_mappings[$sanitized_old_login] = $user_id;
				} else {
						// set a warning in the importer class, and include the
						// WP_Error object emitted by wp_insert_user() as data.
					$this->warnings[] = new WP_Error(
							'import_process_author_mappings_warning',
							sprintf(
									__(
											'Failed to create new user for %s. Their posts will be attributed to the current user.',
											'wordpress-importer' ),
									esc_html(
											$this->authors[$old_login]['author_display_name'] ) ),
							$user_id );
					// really wish WP_Error were like an exception and that we
					// could use try...catch...finally.
				}
			}

			/*
			 * Failsafe: if the user_id was invalid, default to the current user.
			 *
			 * Cron note: can't use logged in user; instead default to the user
			 * ID stored in params, which should be the admin user who triggered
			 * the import. It is possible to override this from an external
			 * scope by calling $importer->set_param('admin_user_id', $a_value)
			 * after do_parse().
			 */
			if ( ! isset( $this->processed_author_mappings[$sanitized_old_login] ) ) {
				if ( $old_id )
					$this->processed_authors[$old_id] = (int) $this->get_param(
							'admin_user_id', 0 );
				$this->processed_author_mappings[$sanitized_old_login] = (int) $this->get_param(
						'admin_user_id', 0 );
			}
		}

		/*
		 * At this point, theoretically all authors should be mapped. However,
		 * emit a warning if the number of mapped authors is fewer than the
		 * number of authors identified from the import data.
		 */
		if ( count( $this->processed_author_mappings ) < count( $this->authors ) ) {
			$this->warnings[] = new WP_Error(
					'import_process_author_mappings_warning',
					__(
							'Some authors may not have been attributed correctly.',
							'wordpress-importer' ) );
		}

		$this->advance();
		return true;
	}

	/**
	 * Creates new categories based on those used in the import data.
	 *
	 * Depends on do_parse(), which sets $this->categories.
	 */
	protected function process_categories() {
		// allow some categories to be filtered out from import
		$this->categories = apply_filters( 'wp_import_categories',
				$this->categories );

		if ( empty( $this->categories ) ) {
			$this->advance();
			return true;
		}

		foreach ( $this->categories as $cat ) {
			// if the category already exists leave it alone
			$term_id = term_exists( $cat['category_nicename'], 'category' );
			if ( $term_id ) {
				if ( is_array( $term_id ) )
					$term_id = $term_id['term_id'];
				if ( isset( $cat['term_id'] ) )
					$this->processed_terms[intval( $cat['term_id'] )] = (int) $term_id;
				continue;
			}

			$category_parent = empty( $cat['category_parent'] ) ? 0 : category_exists(
					$cat['category_parent'] );
			$category_description = isset( $cat['category_description'] ) ? $cat['category_description'] : '';
			$catarr = array(
					'category_nicename' => $cat['category_nicename'],
					'category_parent' => $category_parent,
					'cat_name' => $cat['cat_name'],
					'category_description' => $category_description
			);

			$id = wp_insert_category( $catarr );
			if ( ! is_wp_error( $id ) ) {
				if ( isset( $cat['term_id'] ) )
					$this->processed_terms[intval( $cat['term_id'] )] = $id;
			} else {
				// set a warning in the importer class, and include the
				// WP_Error object emitted by wp_insert_category() as data.
				$this->warnings[] = new WP_Error(
						'import_process_categories_warning',
						sprintf(
								__( 'Failed to import category %s',
										'wordpress-importer' ),
								esc_html( $cat['category_nicename'] ) ), $id );
				continue;
			}
		}

		// saves us some memory
		unset( $this->categories );

		$this->advance();
		return true;
	}

	/**
	 * Creates new tags based on those used in the import data.
	 *
	 * Depends on do_parse(), which sets $this->tags. Very similar in structure
	 * and function to process_categories().
	 */
	protected function process_tags() {
		$this->tags = apply_filters( 'wp_import_tags', $this->tags );

		if ( empty( $this->tags ) ) {
			$this->advance();
			return true;
		}

		foreach ( $this->tags as $tag ) {
			// if the tag already exists leave it alone
			$term_id = term_exists( $tag['tag_slug'], 'post_tag' );
			if ( $term_id ) {
				if ( is_array( $term_id ) )
					$term_id = $term_id['term_id'];
				if ( isset( $tag['term_id'] ) )
					$this->processed_terms[intval( $tag['term_id'] )] = (int) $term_id;
				continue;
			}

			$tag_desc = isset( $tag['tag_description'] ) ? $tag['tag_description'] : '';
			$tagarr = array(
					'slug' => $tag['tag_slug'],
					'description' => $tag_desc
			);

			$id = wp_insert_term( $tag['tag_name'], 'post_tag', $tagarr );
			if ( ! is_wp_error( $id ) ) {
				if ( isset( $tag['term_id'] ) )
					$this->processed_terms[intval( $tag['term_id'] )] = $id['term_id'];
			} else {
					// set a warning in the importer class, and include the
					// WP_Error object emitted by wp_insert_category() as data.
				$this->warnings[] = new WP_Error(
						'import_process_tags_warning',
						sprintf(
								__( 'Failed to import post tag %s',
										'wordpress-importer' ),
								esc_html( $tag['tag_name'] ) ), $id );
				continue;
			}
		}

		unset( $this->tags );

		$this->advance();
		return true;
	}

	/**
	 * Creates new terms based on those used in the import data.
	 *
	 * Depends on do_parse(), which sets $this->terms. Very similar in structure
	 * and function to process_categories() and process_tags().
	 */
	protected function process_terms() {
		$this->terms = apply_filters( 'wp_import_terms', $this->terms );

		if ( empty( $this->terms ) ) {
			$this->advance();
			return true;
		}

		foreach ( $this->terms as $term ) {
			// if the term already exists in the correct taxonomy leave it alone
			$term_id = term_exists( $term['slug'], $term['term_taxonomy'] );
			if ( $term_id ) {
				if ( is_array( $term_id ) )
					$term_id = $term_id['term_id'];
				if ( isset( $term['term_id'] ) )
					$this->processed_terms[intval( $term['term_id'] )] = (int) $term_id;
				continue;
			}

			if ( empty( $term['term_parent'] ) ) {
				$parent = 0;
			} else {
				$parent = term_exists( $term['term_parent'],
						$term['term_taxonomy'] );
				if ( is_array( $parent ) )
					$parent = $parent['term_id'];
			}
			$description = isset( $term['term_description'] ) ? $term['term_description'] : '';
			$termarr = array(
					'slug' => $term['slug'],
					'description' => $description,
					'parent' => intval( $parent )
			);

			$id = wp_insert_term( $term['term_name'], $term['term_taxonomy'],
					$termarr );
			if ( ! is_wp_error( $id ) ) {
				if ( isset( $term['term_id'] ) )
					$this->processed_terms[intval( $term['term_id'] )] = $id['term_id'];
			} else {
				// set a warning in the importer class, and include the
				// WP_Error object emitted by wp_insert_term() as data.
				$this->warnings[] = new WP_Error(
						'import_process_term_warning',
						sprintf(
								__( 'Failed to import %s %s',
										'wordpress-importer' ),
								esc_html( $term['term_taxonomy'] ),
								esc_html( $term['term_name'] ) ), $id );
				continue;
			}
		} // foreach

		unset( $this->terms );
		$this->advance();
		return true;
	}

	/**
	 * Fetches attachments if permitted and inserts them with metadata.
	 *
	 * @param array $post
	 * @param string $url
	 * @return int
	 */
	protected function process_attachment( $post, $url ) {
		if ( ! $this->get_param( 'fetch_attachments', true ) ||
				 ! $this->is_allowed_attachment_fetch() ) {
		    $this->warnings[] = new WP_Error( 'attachment_processing_error',
		        __( 'Fetching attachments is not enabled', 'wordpress-importer' ) );
		    return 0;
		}

		// if the URL is absolute, but does not contain address, then upload it assuming base_site_url
		if ( preg_match( '|^/[\w\W]+$|', $url ) )
			$url = rtrim( $this->get_param( 'base_url', '' ), '/' ) . $url;

		$upload = $this->fetch_remote_file( $url, $post );
		if ( is_wp_error( $upload ) )
		    return $upload;

		if ( $info = wp_check_filetype( $upload['file'] ) )
		    $post['post_mime_type'] = $info['type'];
		else
		    return new WP_Error( 'attachment_processing_error', __('Invalid file type', 'wordpress-importer') );

		$post['guid'] = $upload['url'];

		// as per wp-admin/includes/upload.php
		$post_id = wp_insert_attachment( $post, $upload['file'] );
		wp_update_attachment_metadata( $post_id, wp_generate_attachment_metadata( $post_id, $upload['file'] ) );

		// remap resized image URLs, works by stripping the extension and remapping the URL stub.
		if ( preg_match( '!^image/!', $info['type'] ) ) {
		    $parts = pathinfo( $url );
		    $name = basename( $parts['basename'], ".{$parts['extension']}" ); // PATHINFO_FILENAME in PHP 5.2

		    $parts_new = pathinfo( $upload['url'] );
		    $name_new = basename( $parts_new['basename'], ".{$parts_new['extension']}" );

		    $this->processed_url_remap[$parts['dirname'] . '/' . $name] = $parts_new['dirname'] . '/' . $name_new;
		}

		return $post_id;
	}

	/**
	 * Attempts to create a new menu item from import data.
	 *
	 * Fails for draft, orphaned menu items and those without an associated
	 * nav_menu or an invalid nav_menu term. If the post type or term object
	 * which the menu item represents doesn't exist, the menu item will not be
	 * imported (waits until the end of the import to retry again before
	 * discarding).
	 *
	 * @param array $item Menu item details from WXR file
	 * @return int|WP_Error
	 */
	protected function process_menu_item( $item ) {
		// skip draft, orphaned menu items
		if ( 'draft' == $item['status'] )
			return 0;

		$menu_slug = false;
		if ( isset($item['terms']) ) {
			// loop through terms, assume first nav_menu term is correct menu
			foreach ( $item['terms'] as $term ) {
				if ( 'nav_menu' == $term['domain'] ) {
					$menu_slug = $term['slug'];
					break;
				}
			}
		}

		// no nav_menu term associated with this menu item
		if ( ! $menu_slug ) {
			$this->warnings[] = new WP_Error(
					'import_process_menu_item_slug_warning',
					__( 'Menu item skipped due to missing menu slug',
							'wordpress-importer' ) );
			return 0;
		}

		$menu_id = term_exists( $menu_slug, 'nav_menu' );
		if ( ! $menu_id ) {
			$this->warnings[] = new WP_Error(
					'import_process_menu_item_slug_warning',
					sprintf(
							__(
									'Menu item skipped due to invalid menu slug: %s',
									'wordpress-importer' ),
							esc_html( $menu_slug ) ) );
			return 0;
		} else {
			$menu_id = is_array( $menu_id ) ? $menu_id['term_id'] : $menu_id;
		}

		foreach ( $item['postmeta'] as $meta )
			$$meta['key'] = $meta['value'];

		if ( 'taxonomy' == $_menu_item_type && isset( $this->processed_terms[intval($_menu_item_object_id)] ) ) {
			$_menu_item_object_id = $this->processed_terms[intval($_menu_item_object_id)];
		} else if ( 'post_type' == $_menu_item_type && isset( $this->processed_posts[intval($_menu_item_object_id)] ) ) {
			$_menu_item_object_id = $this->processed_posts[intval($_menu_item_object_id)];
		} else if ( 'custom' != $_menu_item_type ) {
			// associated object is missing or not imported yet, we'll retry later
			$this->processed_menu_missing_items[] = $item;
			return;
		}

		if ( isset( $this->processed_menu_items[intval($_menu_item_menu_item_parent)] ) ) {
			$_menu_item_menu_item_parent = $this->processed_menu_items[intval($_menu_item_menu_item_parent)];
		} else if ( $_menu_item_menu_item_parent ) {
			$this->processed_menu_orphan_items[intval($item['post_id'])] = (int) $_menu_item_menu_item_parent;
			$_menu_item_menu_item_parent = 0;
		}

		// wp_update_nav_menu_item expects CSS classes as a space separated string
		$_menu_item_classes = maybe_unserialize( $_menu_item_classes );
		if ( is_array( $_menu_item_classes ) )
			$_menu_item_classes = implode( ' ', $_menu_item_classes );

		$args = array(
			'menu-item-object-id' => $_menu_item_object_id,
			'menu-item-object' => $_menu_item_object,
			'menu-item-parent-id' => $_menu_item_menu_item_parent,
			'menu-item-position' => intval( $item['menu_order'] ),
			'menu-item-type' => $_menu_item_type,
			'menu-item-title' => $item['post_title'],
			'menu-item-url' => $_menu_item_url,
			'menu-item-description' => $item['post_content'],
			'menu-item-attr-title' => $item['post_excerpt'],
			'menu-item-target' => $_menu_item_target,
			'menu-item-classes' => $_menu_item_classes,
			'menu-item-xfn' => $_menu_item_xfn,
			'menu-item-status' => $item['status']
		);

		$id = wp_update_nav_menu_item( $menu_id, 0, $args );
		if ( $id && ! is_wp_error( $id ) )
			$this->processed_menu_items[intval($item['post_id'])] = (int) $id;

		return $id;
	}

	/**
	 * Creates new posts based on import data.
	 *
	 * Posts marked as having a parent which doesn't exist will become top level
	 * items. Doesn't create a new post if: the post type doesn't exist, the
	 * given post ID is already noted as imported or a post with the same title
	 * and date already exists. Note that new/updated terms, comments and meta
	 * are imported for the last of the above.
	 */
	protected function process_posts() {
		if ( $this->get_param( 'filtered_wp_import_posts', false ) === false ) {
			// only filter once!
			$this->posts = apply_filters( 'wp_import_posts', $this->posts );
			$this->set_param( 'filtered_wp_import_posts', true );
		}

		// use an iterator object with which we can track state
		$iterator = new ArrayIterator( $this->posts );

		foreach ( $iterator as $post ) {
			$post = apply_filters( 'wp_import_post_data_raw', $post );
			if ( ! post_type_exists( $post['post_type'] ) ) {
				$error_data = array(
						'title' => esc_html( $post['post_title'] ),
						'post_type' => esc_html( $post['post_type'] )
				);
				$this->warnings[] = new WP_Error(
						'import_process_posts_post_type_warning',
						sprintf(
								__(
										'Failed to import &#8220;%s&#8221;: Invalid post type %s',
										'wordpress-importer' ),
								$error_data['title'], $error_data['post_type'] ),
						$error_data );
				do_action( 'wp_import_post_exists', $post );
				continue;
			}

			// skip already-processed items
			if ( isset( $this->processed_posts[$post['post_id']] ) &&
					 ! empty( $post['post_id'] ) )
				continue;

			if ( $post['status'] == 'auto-draft' )
				continue;

			if ( 'nav_menu_item' == $post['post_type'] ) {
				$this->process_menu_item( $post );
				continue;
			}

			$post_type_object = get_post_type_object( $post['post_type'] );

			$post_exists = post_exists( $post['post_title'], '',
					$post['post_date'] );
			if ( $post_exists &&
					 get_post_type( $post_exists ) == $post['post_type'] ) {
				$this->warnings[] = new WP_Error(
						'import_process_posts_post_exists_warning',
						sprintf(
								__( '%s &#8220;%s&#8221; already exists.',
										'wordpress-importer' ),
								$post_type_object->labels->singular_name,
								esc_html( $post['post_title'] ) ),
						array(
								'post_type' => $post_type_object->labels->singular_name,
								'title' => esc_html( $post['post_title'] )
						) );
				$comment_post_ID = $post_id = $post_exists;
			} else {
				$post_parent = (int) $post['post_parent'];
				if ( $post_parent ) {
					// if we already know the parent, map it to the new local ID
					if ( isset( $this->processed_posts[$post_parent] ) ) {
						$post_parent = $this->processed_posts[$post_parent];
						// otherwise record the parent for later
					} else {
						$this->processed_post_orphans[intval(
								$post['post_id'] )] = $post_parent;
						$post_parent = 0;
					}
				}

				// map the post author
				$author = sanitize_user( $post['post_author'], true );
				if ( isset( $this->processed_author_mappings[$author] ) )
					$author = $this->processed_author_mappings[$author];
				else
					$author = (int) $this->get_param( 'admin_user_id', 0 );

				$postdata = array(
						'import_id' => $post['post_id'],
						'post_author' => $author,
						'post_date' => $post['post_date'],
						'post_date_gmt' => $post['post_date_gmt'],
						'post_content' => $post['post_content'],
						'post_excerpt' => $post['post_excerpt'],
						'post_title' => $post['post_title'],
						'post_status' => $post['status'],
						'post_name' => $post['post_name'],
						'comment_status' => $post['comment_status'],
						'ping_status' => $post['ping_status'],
						'guid' => $post['guid'],
						'post_parent' => $post_parent,
						'menu_order' => $post['menu_order'],
						'post_type' => $post['post_type'],
						'post_password' => $post['post_password']
				);

				$original_post_ID = $post['post_id'];
				$postdata = apply_filters( 'wp_import_post_data_processed',
						$postdata, $post );

				/*
				 * @todo decouple attachment processing from post processing
				 */
				if ( 'attachment' == $postdata['post_type'] ) {
					$remote_url = ! empty( $post['attachment_url'] ) ? $post['attachment_url'] : $post['guid'];

					// try to use _wp_attached file for upload folder placement
					// to ensure the same location as the export site
					// e.g. location is 2003/05/image.jpg but the attachment
					// post_date is 2010/09, see media_handle_upload()
					$postdata['upload_date'] = $post['post_date'];
					if ( isset( $post['postmeta'] ) ) {
						foreach ( $post['postmeta'] as $meta ) {
							if ( $meta['key'] == '_wp_attached_file' ) {
								if ( preg_match( '%^[0-9]{4}/[0-9]{2}%',
										$meta['value'], $matches ) )
									$postdata['upload_date'] = $matches[0];
								break;
							}
						}
					}

					$comment_post_ID = $post_id = $this->process_attachment(
							$postdata, $remote_url );
				} else {
					$comment_post_ID = $post_id = wp_insert_post( $postdata,
							true );
					do_action( 'wp_import_insert_post', $post_id,
							$original_post_ID, $postdata, $post );
				}

				if ( is_wp_error( $post_id ) ) {
					$this->warnings[] = new WP_Error(
							'import_process_posts_failure_warning',
							sprintf(
									__(
											'Failed to import %s &#8220;%s&#8221;',
											'wordpress-importer' ),
									$post_type_object->labels->singular_name,
									esc_html( $post['post_title'] ) ), $post_id );
					continue;
				}

				if ( $post['is_sticky'] == 1 )
					stick_post( $post_id );
			}

			// map pre-import ID to local ID
			$this->processed_posts[intval( $post['post_id'] )] = (int) $post_id;

			if ( ! isset( $post['terms'] ) )
				$post['terms'] = array();

			$post['terms'] = apply_filters( 'wp_import_post_terms',
					$post['terms'], $post_id, $post );

			// add categories, tags and other terms
			if ( ! empty( $post['terms'] ) ) {
				$terms_to_set = array();
				foreach ( $post['terms'] as $term ) {
					// back compat with WXR 1.0 map 'tag' to 'post_tag'
					$taxonomy = ('tag' == $term['domain']) ? 'post_tag' : $term['domain'];
					$term_exists = term_exists( $term['slug'], $taxonomy );
					$term_id = is_array( $term_exists ) ? $term_exists['term_id'] : $term_exists;
					if ( ! $term_id ) {
						$t = wp_insert_term( $term['name'], $taxonomy,
								array(
										'slug' => $term['slug']
								) );
						if ( ! is_wp_error( $t ) ) {
							$term_id = $t['term_id'];
							do_action( 'wp_import_insert_term', $t, $term,
									$post_id, $post );
						} else {
							$this->warnings[] = new WP_Error(
									'import_process_term_warning',
									sprintf(
											__( 'Failed to import %s %s',
													'wordpress-importer' ),
											esc_html( $taxonomy ),
											esc_html( $term['name'] ) ), $t );
							do_action( 'wp_import_insert_term_failed', $t,
									$term, $post_id, $post );
							continue;
						}
					}
					$terms_to_set[$taxonomy][] = intval( $term_id );
				}

				foreach ( $terms_to_set as $tax => $ids ) {
					$tt_ids = wp_set_post_terms( $post_id, $ids, $tax );
					do_action( 'wp_import_set_post_terms', $tt_ids, $ids,
							$tax, $post_id, $post );
				}
				unset( $post['terms'], $terms_to_set );
			}

			if ( ! isset( $post['comments'] ) )
				$post['comments'] = array();

			$post['comments'] = apply_filters( 'wp_import_post_comments',
					$post['comments'], $post_id, $post );

			// add/update comments
			if ( ! empty( $post['comments'] ) ) {
				$num_comments = 0;
				$inserted_comments = array();
				foreach ( $post['comments'] as $comment ) {
					$comment_id = $comment['comment_id'];
					$newcomments[$comment_id]['comment_post_ID'] = $comment_post_ID;
					$newcomments[$comment_id]['comment_author'] = $comment['comment_author'];
					$newcomments[$comment_id]['comment_author_email'] = $comment['comment_author_email'];
					$newcomments[$comment_id]['comment_author_IP'] = $comment['comment_author_IP'];
					$newcomments[$comment_id]['comment_author_url'] = $comment['comment_author_url'];
					$newcomments[$comment_id]['comment_date'] = $comment['comment_date'];
					$newcomments[$comment_id]['comment_date_gmt'] = $comment['comment_date_gmt'];
					$newcomments[$comment_id]['comment_content'] = $comment['comment_content'];
					$newcomments[$comment_id]['comment_approved'] = $comment['comment_approved'];
					$newcomments[$comment_id]['comment_type'] = $comment['comment_type'];
					$newcomments[$comment_id]['comment_parent'] = $comment['comment_parent'];
					$newcomments[$comment_id]['commentmeta'] = isset(
							$comment['commentmeta'] ) ? $comment['commentmeta'] : array();
					if ( isset(
							$this->processed_authors[$comment['comment_user_id']] ) )
						$newcomments[$comment_id]['user_id'] = $this->processed_authors[$comment['comment_user_id']];
				}
				ksort( $newcomments );

				foreach ( $newcomments as $key => $comment ) {
					// if this is a new post we can skip the comment_exists()
					// check
					if ( ! $post_exists || ! comment_exists(
							$comment['comment_author'],
							$comment['comment_date'] ) ) {
						if ( isset(
								$inserted_comments[$comment['comment_parent']] ) )
							$comment['comment_parent'] = $inserted_comments[$comment['comment_parent']];
						$comment = wp_filter_comment( $comment );
						$inserted_comments[$key] = wp_insert_comment(
								$comment );
						do_action( 'wp_import_insert_comment',
								$inserted_comments[$key], $comment,
								$comment_post_ID, $post );

						foreach ( $comment['commentmeta'] as $meta ) {
							$value = maybe_unserialize( $meta['value'] );
							add_comment_meta( $inserted_comments[$key],
									$meta['key'], $value );
						}

						$num_comments ++;
					}
				}
				unset( $newcomments, $inserted_comments, $post['comments'] );
			}

			if ( ! isset( $post['postmeta'] ) )
				$post['postmeta'] = array();

			$post['postmeta'] = apply_filters( 'wp_import_post_meta',
					$post['postmeta'], $post_id, $post );

			// add/update post meta
			if ( ! empty( $post['postmeta'] ) ) {
				foreach ( $post['postmeta'] as $meta ) {
					$key = apply_filters( 'import_post_meta_key',
							$meta['key'], $post_id, $post );
					$value = false;

					if ( '_edit_last' == $key ) {
						if ( isset(
								$this->processed_authors[intval(
										$meta['value'] )] ) )
							$value = $this->processed_authors[intval(
									$meta['value'] )];
						else
							$key = false;
					}

					if ( $key ) {
						// export gets meta straight from the DB so could have a
						// serialized string
						if ( ! $value )
							$value = maybe_unserialize( $meta['value'] );

						add_post_meta( $post_id, $key, $value );
						do_action( 'import_post_meta', $post_id, $key,
								$value );

						// if the post has a featured image, take note of this
						// in case of remap
						if ( '_thumbnail_id' == $key )
							$this->processed_featured_images[$post_id] = (int) $value;
					}
				}
			}

			// stop execution if we've reached the end of this cron run's
			// limit
			if ( ! $this->have_time() )
				break;
		} // foreach

		// only advance after all posts have gone through the foreach
		if(! $iterator->valid()) {
			// done
			$this->set_param( 'count_posts', count( $this->posts ) );
			unset( $this->posts );
			$this->advance();
			return true;
		}

		return false;
	}

	/**
	 * Attempts to associate posts & menu items with previously missing parents.
	 *
	 * An imported post's parent may not have been imported when it was first
	 * created, so try again. Also for child menu items and menu items
	 * which were missing the object (e.g. post) they represent in the menu.
	 */
	protected function process_parents() {
		// find parents for post orphans
		foreach ( $this->processed_post_orphans as $child_id => $parent_id ) {
			$local_child_id = $local_parent_id = false;
			if ( isset( $this->processed_posts[$child_id] ) )
				$local_child_id = $this->processed_posts[$child_id];
			if ( isset( $this->processed_posts[$parent_id] ) )
				$local_parent_id = $this->processed_posts[$parent_id];
			if ( $local_child_id && $local_parent_id ) {
				// this used to use $wpdb->update(), but this would be
				// preferable for a self-contained importer
				$update = wp_update_post(
						array(
								'ID' => (int) $local_child_id,
								'post_parent' => (int) $local_parent_id
						), true );
				if ( is_wp_error( $update ) )
					$this->warnings[] = new WP_Error(
							'import_process_parents_post_warning',
							sprintf(
									__(
											'Failed to update parent on post orphan with ID %d',
											'wordpress-importer' ),
									$local_child_id ), $update );
			}
		}

		// all other posts/terms are imported, retry menu items with missing
		// associated object
		foreach ( $this->processed_menu_missing_items as $item )
			$this->process_menu_item( $item );

		// find parents for menu item orphans
		foreach ( $this->processed_menu_orphan_items as $child_id => $parent_id ) {
			$local_child_id = $local_parent_id = 0;
			if ( isset( $this->processed_menu_items[$child_id] ) )
				$local_child_id = $this->processed_menu_items[$child_id];
			if ( isset( $this->processed_menu_items[$parent_id] ) )
				$local_parent_id = $this->processed_menu_items[$parent_id];

			if ( $local_child_id && $local_parent_id )
				update_post_meta( $local_child_id,
						'_menu_item_menu_item_parent', (int) $local_parent_id );
		}

		$this->advance();
		return true;
	}

	/**
	 * Remaps attachment URLs to the new location.
	 *
	 * @todo Use some form of WP_Relocate instead of directly querying SQL
	 * @global $wpdb
	 * @return boolean True if completed, false if the process must continue in
	 *         the next cron run
	 */
	protected function process_urls() {
		global $wpdb;

		if ( $this->get_param( 'sorted_processed_url_remap', false ) === false ) {
			// only sort once!
			uksort( $this->processed_url_remap,
					array(
							get_class( $this ),
							'compare_strlen'
					) );
			$this->set_param( 'sorted_processed_url_remap', true );
		}

		// use an iterator object with which we can track state
		$iterator = new ArrayIterator( $this->processed_url_remap );

		// make sure we do the longest urls first, in case one is a
		// substring of another
		foreach ( $iterator as $from_url => $to_url ) {
			// remap urls in post_content
			$wpdb->query(
					$wpdb->prepare(
							"UPDATE {$wpdb->posts} SET post_content = REPLACE(post_content, %s, %s)",
							$from_url, $to_url ) );
			// remap enclosure urls
			$result = $wpdb->query(
					$wpdb->prepare(
							"UPDATE {$wpdb->postmeta} SET meta_value = REPLACE(meta_value, %s, %s) WHERE meta_key='enclosure'",
							$from_url, $to_url ) );

			// unset this key so we don't do it again next time
			unset( $this->processed_url_remap[$from_url] );

			// stop execution if we've reached the end of this cron run's
			// limit
			if ( ! $this->have_time() )
				break;
		}

		// only unset after all posts have gone through the foreach
		if ( ! $iterator->valid() ) {
			// done
			$this->advance();
			return true;
		}

		return false;
	}

	/**
	 * Updates _thumbnail_id meta to new, imported attachment IDs.
	 */
	protected function process_featured_images() {
	    // cycle through posts that have a featured image
	    foreach ( $this->processed_featured_images as $post_id => $value ) {
	        if ( isset( $this->processed_posts[$value] ) ) {
	            $new_id = $this->processed_posts[$value];
	            // only update if there's a difference
	            if ( $new_id != $value )
	                update_post_meta( $post_id, '_thumbnail_id', $new_id );
	        }
	    }

	    $this->advance();
	    return true;
	}

	/**
	 * Attempt to download a remote file attachment
	 *
	 * @param string $url
	 *        	URL of item to fetch
	 * @param array $post
	 *        	Attachment details
	 * @return array WP_Error file location details on success, WP_Error
	 *         otherwise
	 */
	public function fetch_remote_file( $url, $post ) {
		// extract the file name and extension from the url
		$file_name = basename( $url );

		// get placeholder file in the upload dir with a unique, sanitized
		// filename
		$upload = wp_upload_bits( $file_name, 0, '', $post['upload_date'] );
		if ( $upload['error'] )
			return new WP_Error( 'upload_dir_error', $upload['error'] );

			// fetch the remote url and write it to the placeholder file
		$headers = wp_get_http( $url, $upload['file'] );

		// request failed
		if ( ! $headers ) {
			@unlink( $upload['file'] );
			return new WP_Error( 'import_file_error',
					__( 'Remote server did not respond',
							'wordpress-importer' ) );
		}

		// make sure the fetch was successful
		if ( $headers['response'] != '200' ) {
			@unlink( $upload['file'] );
			return new WP_Error( 'import_file_error',
					sprintf(
							__(
									'Remote server returned error response %1$d %2$s',
									'wordpress-importer' ),
							esc_html( $headers['response'] ),
							get_status_header_desc( $headers['response'] ) ) );
		}

		$filesize = filesize( $upload['file'] );

		if ( isset( $headers['content-length'] ) &&
				 $filesize != $headers['content-length'] ) {
			@unlink( $upload['file'] );
			return new WP_Error( 'import_file_error',
					__( 'Remote file is incorrect size',
							'wordpress-importer' ) );
		}

		if ( 0 == $filesize ) {
			@unlink( $upload['file'] );
			return new WP_Error( 'import_file_error',
					__( 'Zero size file downloaded', 'wordpress-importer' ) );
		}

		$max_size = (int) $this->get_max_attachment_size();
		if ( ! empty( $max_size ) && $filesize > $max_size ) {
			@unlink( $upload['file'] );
			return new WP_Error( 'import_file_error',
					sprintf(
							__( 'Remote file is too large, limit is %s',
									'wordpress-importer' ),
							size_format( $max_size ) ) );
		}

		// keep track of the old and new urls so we can substitute them later
		$this->processed_url_remap[$url] = $upload['url'];
		$this->processed_url_remap[$post['guid']] = $upload['url'];
		// r13735, really needed?
		// ^ keep track of the destination if the remote url is redirected
		// somewhere else

		if ( isset( $headers['x-final-location'] ) &&
				 $headers['x-final-location'] != $url )
			$this->processed_url_remap[$headers['x-final-location']] = $upload['url'];

		return $upload;
	}

	/**
	 * Decide if the given meta key maps to information we will want to import
	 *
	 * @param string $key
	 *        	The meta key to check
	 * @return string|bool key if we do want to import, false if not
	 */
	public function is_valid_meta_key( $key ) {
		// skip attachment metadata since we'll regenerate it from scratch
		// skip _edit_lock as not relevant for import
		if ( in_array( $key,
				array(
						'_wp_attached_file',
						'_wp_attachment_metadata',
						'_edit_lock'
				) ) )
			return false;
		return $key;
	}

	/**
	 * Decide whether or not the importer is allowed to create users. Default is
	 * true, can be filtered via import_allow_create_users
	 *
	 * @return bool True if creating users is allowed
	 */
	public function is_allowed_user_creation() {
		return apply_filters( 'import_allow_create_users', true );
	}

	/**
	 * Retrieves whether the importer should attempt to download attachment
	 * files.
	 *
	 * Default is true, can be filtered via
	 * import_allow_fetch_attachments. The choice made at the import options
	 * screen must also be true; false here hides that checkbox.
	 *
	 * @return bool True if downloading attachments is allowed
	 */
	public function is_allowed_attachment_fetch() {
		return apply_filters( 'import_allow_fetch_attachments', true );
	}

	/**
	 * Retrieves the maximum file size for downloaded attachments.
	 *
	 * Default is 0 (unlimited), can be filtered via
	 * import_attachment_size_limit.
	 *
	 * @return int Maximum attachment file size to import
	 */
	public function get_max_attachment_size() {
		return apply_filters( 'import_attachment_size_limit', 0 );
	}

	/**
	 * Retrieves the fatal error.
	 *
	 * @return WP_Error|null
	 */
	public function get_error() {
		return $this->error;
	}

	/**
	 * Retrieves all non-fatal warnings.
	 *
	 * @return WP_Error[]
	 */
	public function get_warnings() {
		return $this->warnings;
	}

	/**
	 * Retrieves the current step.
	 *
	 * @return string|WP_Error
	 */
	public function get_step() {
		if ( ! isset( $this->wxr_file ) ) {
		    // cannot set user options before WXR data is parsed
		    return new WP_Error( 'import_step_error',
		        __( 'Cannot get step without parsed data',
		            'wordpress-importer' ) );
		}
		return $this->progress[$this->wxr_file];
	}

	/**
	 * Returns the version of the loaded WXR file.
	 *
	 * @return float
	 */
	public function get_version() {
		return $this->version;
	}

	/**
	 * Retrieves the authors from the WXR file.
	 *
	 * Necessary for UI interface.
	 *
	 * @return array
	 */
	public function get_authors() {
		// runs process_authors() once if it has never been done
		if ( empty( $this->authors ) ) {
			// explicitly tell process_authors() NOT to advance
			$this->process_authors(false);

			// pass through error if there was one, then clear it
			if ( is_wp_error( $this->error ) ) {
				$error = $this->error;
				unset($this->error);
				return $error;
			}
		}

		return $this->authors;
	}

	/**
	 * Stores parameters typically used for user input.
	 *
	 * Previously, some of these params would have been directly accessed with
	 * $_POST; now they are stored.
	 *
	 * @param string $name
	 * @param mixed $value
	 * @return boolean|WP_Error if unsuccessful, an error object
	 */
	public function set_param ( $name, $value ) {
		if ( ! isset( $this->wxr_file ) ) {
			// cannot set user options before WXR data is parsed
			return new WP_Error( 'import_param_set_error',
					__( 'Cannot set param without parsed data',
							'wordpress-importer' ) );
		}
		if ( ! is_array( $this->params[$this->wxr_file] ) )
			$this->params[$this->wxr_file] = array();
		$this->params[$this->wxr_file][$name] = $value;

		return true;
	}

	/**
	 * Unsets a parameter.
	 *
	 * @param string $name
	 * @return boolean|WP_Error
	 */
	public function unset_param ( $name ) {
		if ( ! isset( $this->wxr_file ) ) {
		    // cannot set user options before WXR data is parsed
		    return new WP_Error( 'import_param_unset_error',
		        __( 'Cannot unset param without parsed data',
		            'wordpress-importer' ) );
		}
		if ( ! is_array( $this->params[$this->wxr_file] ) )
		    $this->params[$this->wxr_file] = array();
		elseif(key_exists($name, $this->params[$this->wxr_file]))
			unset($this->params[$this->wxr_file][$name]);

		return true;
	}

	/**
	 * Retrieves parameters typically used for user input.
	 *
	 * @param string $name
	 * @param mixed $default
	 * @return mixed WP_Error requested parameter, or if unsuccessful, an error
	 *         object
	 */
	public function get_param ( $name, $default = null ) {
		if ( 'wxr_file' == $name )
			return $this->wxr_file;

		if ( ! isset( $this->wxr_file ) ) {
			// cannot get user options before WXR data is parsed
			return new WP_Error( 'import_param_get_error',
					__( 'Cannot get param without parsed data',
							'wordpress-importer' ) );
		}
		if ( isset( $this->params[$this->wxr_file] ) &&
				 isset( $this->params[$this->wxr_file][$name] ) )
			return $this->params[$this->wxr_file][$name];

		return $default;
	}

	/**
	 * Returns all params.
	 *
	 * The first dimension of this array is the WXR filename (for example,
	 * <code>$params['/path/to/wxr.xml']</code>).
	 *
	 * @return array
	 */
	public function get_all_params () {
		return $this->params;
	}

	public function count( $variable ) {
		if ( $this->get_param( 'count_' . $variable, - 1 ) !== - 1 )
		    return $this->get_param( 'count_' . $variable );
		elseif ( property_exists( $this, $variable ) )
			return count( $this->$variable );
		return - 1;
	}

	/**
	 * Subtracts the lengths of two strings.
	 *
	 * @param string $a
	 * @param string $b
	 * @return int The difference in their lengths
	 */
	static function compare_strlen( $a, $b ) {
		return strlen( $b ) - strlen( $a );
	}
}