<?php
/**
 * WP_Importer_Cron base
 *
 * Forked from the Tumblr Importer (http://wordpress.org/plugins/tumblr-importer/)
 * at revision 477917; original work by Otto42 and westi.
 *
 * @package WordPress
 * @subpackage Importer
 */

if ( !class_exists( 'WP_Importer_Cron' ) ) :

	require_once ABSPATH . 'wp-admin/includes/import.php';

	if ( ! class_exists( 'WP_Importer' ) ) {
		$class_wp_importer = ABSPATH . 'wp-admin/includes/class-wp-importer.php';
		if ( file_exists( $class_wp_importer ) ) {
			require_once $class_wp_importer;
		} else {
			return;
		}
	}

	/**
	 * WP_Importer_Cron base class
	 *
	 * Adds cron based functionality to WP_Importer, allowing for step-wise
	 * import processes that occur separate from a page view.
	 *
	 * @since 1.0.0
	 */
	class WP_Importer_Cron extends WP_Importer {

		/**
		 * Timestamp of when this object was constructed.
		 *
		 * @var int
		 */
		protected $_importer_started;

		/**
		 * Instantiates a new cron-based importer object.
		 *
		 * If this method is overrided by a child class, it is crucial that the
		 * same initialization steps as below are done, such as by way of
		 * <code>parent::__construct();</code>
		 */
		function __construct() {
			// start the clock
			$this->_importer_started = time();
			$this->_max_execution_time = 30; // default to 30 seconds of time on the clock
			parent::__construct();

			// add the importer schedule
			add_filter( 'cron_schedules', array( $this, 'importer_schedule' ) );

			// add the importer hook
			add_action( 'wp_cron_importer_hook', array( $this, 'importer_callback' ) );

			// crucial! for preventing cache breaking
			if ( defined( 'DOING_CRON' ) ) {
				wp_suspend_cache_invalidation( true );
				wp_suspend_cache_addition( true );
			}

			// load the variables
			$options = get_option( get_class($this) );

			if ( is_array($options) )
				foreach ( $options as $key => $value )
					$this->$key = $value;
		}

		/**
		 * Checks whether there is time remaining until the execution deadline.
		 *
		 * @return boolean
		 */
		function have_time() {
			if ( time() - $this->_importer_started > $this->_max_execution_time )
				return false;
			return true;
		}

		/**
		 * Registers a per-minute cron schedule.
		 *
		 * This is a filter for cron_schedules.
		 *
		 * @see wp_get_schedules()
		 * @param array $schedules
		 * @return array filtered schedules
		 */
		function importer_schedule( $schedules ) {
			$schedules['everyminute'] = array(
					'interval' => 60,
					'display' => __( 'Once Per Minute' )
			);
			return $schedules;
		}

		/**
		 * Schedules an importer job on the per-minute cron schedule.
		 *
		 * @param callback $function_name
		 * @param array $args optional
		 */
		function schedule_import_job( $function_name, $args = array() ) {
			$this->callback = $function_name;
			wp_schedule_event( time(), 'everyminute', 'wp_cron_importer_hook',
					$args );
		}

		/**
		 * Dispatches a cron event to an importer job or cleans up.
		 *
		 * @param array $args
		 *        	optional
		 */
		function importer_callback( $args = array() ) {
			$args = (array) $args;

			if ( isset( $this->callback ) ) {
				$complete = call_user_func_array(
						array(
								$this,
								$this->callback
						), $args );
			} else {
				// no callback, force the cron job to end
				$complete = true;
			}

			if ( $complete ) {
				// importer is finished, stop the scheduler
				wp_clear_scheduled_hook( 'wp_cron_importer_hook', $args );
			}
		}

		/**
		 * Saves object variables to an option for statefulness.
		 *
		 * On later runs, these variables are loaded from the option during
		 * instantiation.
		 *
		 * @return boolean
		 */
		function save_vars() {
			$vars = get_object_vars( $this );
			foreach ( $vars as $var => $val ) {
				if ( $var[0] == '_' )
					unset( $vars[$var] );
			}
			update_option( get_class( $this ), $vars );

			return ! empty( $vars );
		}
	}

endif; // class_exists