<?php
require getenv( 'WP_TESTS_DIR' ) . '/tests/import/base.php';

/**
 * WordPress_Importer (cron-based) test case.
 */
class WordPress_Importer_TestCase extends WP_Import_UnitTestCase {

	/**
	 * Slightly modified from Tests_Import_Import
	 *
	 * @see Tests_Import_Import::setUp()
	 */
	function setUp() {
		parent::setUp();

		if ( ! defined( 'WP_IMPORTING' ) )
			define( 'WP_IMPORTING', true );

		if ( ! defined( 'WP_LOAD_IMPORTERS' ) )
			define( 'WP_LOAD_IMPORTERS', true );

		add_filter( 'import_allow_create_users', '__return_true' );

		require_once dirname( dirname( __FILE__ ) ) .
				 '/class-wp-importer-cron.php';
		require_once dirname( dirname( __FILE__ ) ) .
				 '/class-wordpress-importer.php';
		require_once dirname( dirname( __FILE__ ) ) .
				 '/class-wxr-parser.php';

		global $wpdb;
		// crude but effective: make sure there's no residual data in the main
		// tables
		foreach ( array(
				'posts',
				'postmeta',
				'comments',
				'terms',
				'term_taxonomy',
				'term_relationships',
				'users',
				'usermeta'
		) as $table )
			$wpdb->query( "DELETE FROM {$wpdb->$table}" );
	}

	function tearDown() {
		remove_filter( 'import_allow_create_users', '__return_true' );

		parent::tearDown();
	}

	/**
	 * Tests WordPress_Importer->__construct()
	 */
	public function test___construct() {
		$instance = new WordPress_Importer();

		// check for hooks
		$this->assertNotEquals( false,
				has_filter( 'cron_schedules',
						array(
								$instance,
								'importer_schedule'
						) ) );

		$this->assertNotEquals( false,
				has_action( 'wp_cron_importer_hook',
						array(
								$instance,
								'importer_callback'
						) ) );
	}

	public function test_save_vars() {
		$instance = new WordPress_Importer();
		$vars = $instance->save_vars();

		$this->assertTrue( $vars );
		$this->assertNotEmpty(
				$options = get_option( get_class( $instance ) ) );
	}

	public function test_advance() {
		$instance = new WordPress_Importer();

		// hack it into submission; nothing else will function without a
		// properly defined wxr_file
		self::set( $instance, 'wxr_file', 'test-nonexistent.xml' );

		// with no step defined
		$this->assertEquals( 'start', $instance->advance() );

		// non-existent step
		$this->assertWPError( $instance->advance( rand_str( 10 ) ) );

		// if on a terms step, the next step is posts
		$this->assertEquals( 'terms', $instance->advance( 'terms' ) );
		$this->assertEquals( 'terms', $instance->get_step() );
		$this->assertEquals( 'posts', $instance->advance() );

		// valid step
		$this->assertEquals( 'finish', $instance->advance( 'finish' ) );
	}

	public function test_get_error() {
		// clean instance, no errors
		$instance = new WordPress_Importer();

		$this->assertNull( $instance->get_error() );
	}

	public function test_get_warnings() {
		// clean instance, no warnings
		$instance = new WordPress_Importer();

		$this->assertInternalType( 'array', $instance->get_warnings() );
		$this->assertEmpty( $instance->get_warnings() );
	}

	public function test_params() {
		// clean instance, no warnings
		$instance = new WordPress_Importer();
		$wxr_file = 'test-nonexistent.xml';

		// can't set without a WXR file defined
		$this->assertWPError( $instance->set_param( 'name', 'value' ) );

		// hack it into submission; nothing else will function without a
		// properly defined wxr_file
		self::set( $instance, 'wxr_file', $wxr_file );

		$this->assertTrue( $instance->set_param( 'name', 'value' ) );
		$this->assertEquals( 'value', $instance->get_param( 'name' ) );

		$this->assertCount( 1, $instance->get_all_params() );
		$this->assertArrayHasKey( $wxr_file, $instance->get_all_params() );
	}

	/**
	 * @depends test_get_error
	 */
	public function test_process_authors_standalone() {
		$instance = new WordPress_Importer();

		$this->assertFalse( self::call( $instance, 'process_authors' ) );
		$this->assertWPError( $instance->get_error() );

		return $instance;
	}

	/**
	 * @depends test_get_error
	 * @depends test_get_warnings
	 * @depends test_params
	 */
	public function test_process_author_mappings_standalone() {
		$instance = new WordPress_Importer();

		$this->assertFalse(
				self::call( $instance, 'process_author_mappings' ) );
		$this->assertWPError( $instance->get_error() );

		// start afresh
		$admin_user = wp_create_user( 'admin', rand_str( 12 ) );
		$this->assertNotInstanceOf( 'WP_Error', $admin_user );
		$users = array(
				'alice' => $admin_user, // alice will be mapped to user ID whatever
				'bob' => 'john', // bob will be transformed into john
				'eve' => false // eve will be imported as is
				);
		$_i = 0;
		$authors = $mapping = $new = array();
		// each user is either mapped to a given ID, mapped to a new user
		// with given login or imported using details in WXR file
		foreach ( $users as $_user => $_map ) {
			$authors[$_i] = $_user;
			if ( is_int( $_map ) )
				$mapping[$_i] = $_map;
			elseif ( is_string( $_map ) )
				$new[$_i] = $_map;

			$_i ++;
		}
		unset( $_i );

		$instance = new WordPress_Importer();
		// hack it into submission; nothing else will function without a
		// properly defined wxr_file
		self::set( $instance, 'wxr_file', 'test-nonexistent.xml' );
		$this->assertTrue(
				$instance->set_param( 'admin_user_id', $admin_user ) );
		$this->assertTrue( $instance->is_allowed_user_creation() );

		// set the parameters necessary for the process to run
		$this->assertTrue(
				$instance->set_param( 'imported_authors', $authors ) );
		$this->assertTrue( $instance->set_param( 'user_map', $mapping ) );
		$this->assertTrue( $instance->set_param( 'user_new', $new ) );

		// running the mapping process should be successful, no error/warnings
		$this->assertTrue(
				self::call( $instance, 'process_author_mappings' ) );
		$this->assertNull( $instance->get_error() );
		$this->assertCount( 0, $instance->get_warnings() );

		// since this test case is run by itself without the benefit of
		// a predecessor process_authors(), no author IDs will be "processed"
		$this->assertCount( 0, self::get( $instance, 'processed_authors' ) );

		// however, mappings will exist for all provided users
		$this->assertCount( count( $users ),
				$author_mappings = self::get( $instance,
						'processed_author_mappings' ) );
		$this->assertEquals( array_keys( $users ),
				array_keys( $author_mappings ) );
		foreach ( $author_mappings as $username => $map ) {
			$_user = get_user_by( 'id', $map );
			$this->assertInstanceOf( 'WP_User', $_user );
		}

		// ... and, if a mapping was set by user decision, it should match
		$this->assertEquals( current( $mapping ), $author_mappings['alice'] );
		$this->assertEquals( get_user_by( 'login', $users['bob'] )->ID,
				$author_mappings['bob'] );
	}

	/**
	 * @depends test_get_error
	 * @depends test_get_warnings
	 * @depends test_params
	 * @depends test_advance
	 */
	public function test_process_posts_standalone() {
		/*
		 * An empty posts array causes the process to advance
		 */
		$instance = new WordPress_Importer();

		// hack it into submission; nothing else will function without a
		// properly defined wxr_file
		self::set( $instance, 'wxr_file', 'test-nonexistent.xml' );

		self::set( $instance, 'posts', array() );
		$this->assertEquals( 'posts', $instance->advance( 'posts' ) );
		$this->assertTrue( self::call( $instance, 'process_posts' ) );
		$this->assertEmpty( $instance->get_error() );
		$this->assertEmpty( $instance->get_warnings() );
		$this->assertEquals( 'parents',
				current( self::get( $instance, 'progress' ) ) );

		/*
		 * START OVER to test a post with non-existent post type
		 */
		$instance2 = new WordPress_Importer();

		// hack it into submission; nothing else will function without a
		// properly defined wxr_file
		self::set( $instance2, 'wxr_file', 'test-nonexistent.xml' );

		self::set( $instance2, 'posts',
				array(
						array(
								'post_type' => rand_str( 12 )
						)
				) );
		$this->assertEquals( 'posts', $instance2->advance( 'posts' ) );
		$this->assertTrue( self::call( $instance2, 'process_posts' ) );
		$this->assertEmpty( $instance2->get_error() );
		$this->assertNotEmpty( $instance2->get_warnings() );
		$this->assertWPError(current($instance2->get_warnings()));
		$this->assertEquals('import_process_posts_post_type_warning',
			current($instance2->get_warnings())->get_error_code());

	}

	/**
	 * Tiny test case of a valid WXR file, not utilizing cron scheduler.
	 *
	 * @depends test_save_vars
	 * @depends test_advance
	 * @depends test_get_error
	 * @depends test_get_warnings
	 * @depends test_params
	 */
	public function test_wxr_smallest() {
		$instance = new WordPress_Importer();

		// use the WordPress core test case in valid-wxr-1.1.xml
		$authors = array(
				'john',
		);
		$wxr_file = DIR_TESTDATA . '/export/valid-wxr-1.1.xml';
		$this->assertNotInstanceOf( 'WP_Error',
				$parse = $instance->do_parse( $wxr_file ) );

		// set the parameters necessary for the process to run
		$this->assertTrue( $instance->set_param( 'imported_authors', $authors ) );
		$this->assertTrue( $instance->set_param( 'user_new', $authors ) );
		$this->assertEquals( 'start', $instance->advance( 'start' ) );
		$this->assertTrue( $instance->is_allowed_user_creation() );

		// this import file has a custom taxonomy
		register_taxonomy('post_tax', 'post');
		$this->assertTrue(taxonomy_exists('post_tax'));

		// should be done in one cron run
		$this->assertTrue( $instance->do_import() );
		$this->assertNull( $instance->get_error() );

		$instance->do_cleanup();

		// check author was imported correctly
		$user_count = count_users();
		$this->assertEquals( 1, $user_count['total_users'] );

		// check that terms were imported correctly
		$this->assertEquals( 1, wp_count_terms( 'category' ) );
		$this->assertEquals( 1, wp_count_terms( 'post_tag' ) );
		$this->assertEquals( 1, wp_count_terms( 'post_tax' ) );
		$alpha = get_term_by( 'slug', 'alpha', 'category' );
		$this->assertEquals( 0, $alpha->parent );
		$clippable = get_term_by( 'slug', 'clippable', 'post_tag' );
		$this->assertEquals( 0, $clippable->parent );
		$bieup = get_term_by( 'slug', 'bieup', 'post_tax' );
		$this->assertEquals( 0, $bieup->parent );

		// check posts
		$posts = get_posts(
				array(
						'numberposts' => 10,
						'post_type' => 'any',
						'post_status' => 'any',
						'orderby' => 'ID'
				) );
		$this->assertCount( 2, $posts );

		$author = get_user_by( 'login', 'john' );

		// check posts and pages
		$post = $posts[0];
		$this->assertEquals( 'About', $post->post_title );
		$this->assertEquals( 'about', $post->post_name );
		$this->assertEquals( $author->ID, $post->post_author );
		$this->assertEquals( 'page', $post->post_type );
		$this->assertEquals( 'publish', $post->post_status );
		$this->assertEquals( 0, $post->post_parent );
		$this->assertEquals( 'default',
				get_post_meta( $post->ID, '_wp_page_template', true ) );

		$post = $posts[1];
		$this->assertEquals( 'Hello world!', $post->post_title );
		$this->assertEquals( 'hello-world', $post->post_name );
		$this->assertEquals( $author->ID, $post->post_author );
		$this->assertEquals( 'post', $post->post_type );
		$this->assertEquals( 'publish', $post->post_status );
		$this->assertEquals( 0, $post->post_parent );
		$cats = wp_get_post_categories( $post->ID, array( 'fields' => 'all' ) );
		$this->assertEquals( 1, count( $cats ) );
		$this->assertEquals('alpha', $cats[0]->slug );
		$tags = wp_get_post_tags( $post->ID );
		$this->assertEquals( 1, count($tags) );
		$this->assertEquals( 'clippable', $tags[0]->slug );
	}

	/**
	 * Standard test case of a WXR file, not utilizing cron scheduler.
	 *
	 * @depends test_save_vars
	 * @depends test_advance
	 * @depends test_get_error
	 * @depends test_get_warnings
	 * @depends test_params
	 */
	public function test_wxr_standard() {
	    $instance = new WordPress_Importer();

			// use the WordPress core test case
		$authors = array(
				'admin',
				'editor',
				'author'
		);
		$wxr_file = DIR_TESTDATA . '/export/small-export.xml';
		$this->assertNotInstanceOf( 'WP_Error',
				$parse = $instance->do_parse( $wxr_file ) );

		// set the parameters necessary for the process to run
		$this->assertTrue(
				$instance->set_param( 'imported_authors', $authors ) );
		$this->assertEquals( 'start', $instance->advance( 'start' ) );
		$this->assertTrue( $instance->is_allowed_user_creation() );

		// should be done in one cron run
		$this->assertTrue( $instance->do_import() );
		$this->assertNull( $instance->get_error() );
		$this->assertEmpty( $instance->get_warnings() );
		$instance->do_cleanup();

		// ensure that authors were imported correctly
		$user_count = count_users();
		$this->assertEquals( 3, $user_count['total_users'] );
		$admin = get_user_by( 'login', 'admin' );
		$this->assertEquals( 'admin', $admin->user_login );
		$this->assertEquals( 'local@host.null', $admin->user_email );
		$editor = get_user_by( 'login', 'editor' );
		$this->assertEquals( 'editor', $editor->user_login );
		$this->assertEquals( 'editor@example.org', $editor->user_email );
		$this->assertEquals( 'FirstName', $editor->user_firstname );
		$this->assertEquals( 'LastName', $editor->user_lastname );
		$author = get_user_by( 'login', 'author' );
		$this->assertEquals( 'author', $author->user_login );
		$this->assertEquals( 'author@example.org', $author->user_email );

		// check that terms were imported correctly
		$this->assertEquals( 30, wp_count_terms( 'category' ) );
		$this->assertEquals( 3, wp_count_terms( 'post_tag' ) );
		$foo = get_term_by( 'slug', 'foo', 'category' );
		$this->assertEquals( 0, $foo->parent );
		$bar = get_term_by( 'slug', 'bar', 'category' );
		$foo_bar = get_term_by( 'slug', 'foo-bar', 'category' );
		$this->assertEquals( $bar->term_id, $foo_bar->parent );

		// check that posts/pages were imported correctly
		$post_count = wp_count_posts( 'post' );
		$this->assertEquals( 5, $post_count->publish );
		$this->assertEquals( 1, $post_count->private );
		$page_count = wp_count_posts( 'page' );
		$this->assertEquals( 4, $page_count->publish );
		$this->assertEquals( 1, $page_count->draft );
		$comment_count = wp_count_comments();
		$this->assertEquals( 1, $comment_count->total_comments );

		$posts = get_posts(
				array(
						'numberposts' => 20,
						'post_type' => 'any',
						'post_status' => 'any',
						'orderby' => 'ID'
				) );
		$this->assertEquals( 11, count( $posts ) );

		$post = $posts[0];
		$this->assertEquals( 'Many Categories', $post->post_title );
		$this->assertEquals( 'many-categories', $post->post_name );
		$this->assertEquals( $admin->ID, $post->post_author );
		$this->assertEquals( 'post', $post->post_type );
		$this->assertEquals( 'publish', $post->post_status );
		$this->assertEquals( 0, $post->post_parent );
		$cats = wp_get_post_categories( $post->ID );
		$this->assertEquals( 27, count( $cats ) );

		$post = $posts[1];
		$this->assertEquals( 'Non-standard post format', $post->post_title );
		$this->assertEquals( 'non-standard-post-format', $post->post_name );
		$this->assertEquals( $admin->ID, $post->post_author );
		$this->assertEquals( 'post', $post->post_type );
		$this->assertEquals( 'publish', $post->post_status );
		$this->assertEquals( 0, $post->post_parent );
		$cats = wp_get_post_categories( $post->ID );
		$this->assertEquals( 1, count( $cats ) );
		$this->assertTrue( has_post_format( 'aside', $post->ID ) );

		$post = $posts[2];
		$this->assertEquals( 'Top-level Foo', $post->post_title );
		$this->assertEquals( 'top-level-foo', $post->post_name );
		$this->assertEquals( $admin->ID, $post->post_author );
		$this->assertEquals( 'post', $post->post_type );
		$this->assertEquals( 'publish', $post->post_status );
		$this->assertEquals( 0, $post->post_parent );
		$cats = wp_get_post_categories( $post->ID,
				array(
						'fields' => 'all'
				) );
		$this->assertEquals( 1, count( $cats ) );
		$this->assertEquals( 'foo', $cats[0]->slug );

		$post = $posts[3];
		$this->assertEquals( 'Foo-child', $post->post_title );
		$this->assertEquals( 'foo-child', $post->post_name );
		$this->assertEquals( $editor->ID, $post->post_author );
		$this->assertEquals( 'post', $post->post_type );
		$this->assertEquals( 'publish', $post->post_status );
		$this->assertEquals( 0, $post->post_parent );
		$cats = wp_get_post_categories( $post->ID,
				array(
						'fields' => 'all'
				) );
		$this->assertEquals( 1, count( $cats ) );
		$this->assertEquals( 'foo-bar', $cats[0]->slug );

		$post = $posts[4];
		$this->assertEquals( 'Private Post', $post->post_title );
		$this->assertEquals( 'private-post', $post->post_name );
		$this->assertEquals( $admin->ID, $post->post_author );
		$this->assertEquals( 'post', $post->post_type );
		$this->assertEquals( 'private', $post->post_status );
		$this->assertEquals( 0, $post->post_parent );
		$cats = wp_get_post_categories( $post->ID );
		$this->assertEquals( 1, count( $cats ) );
		$tags = wp_get_post_tags( $post->ID );
		$this->assertEquals( 3, count( $tags ) );
		$this->assertEquals( 'tag1', $tags[0]->slug );
		$this->assertEquals( 'tag2', $tags[1]->slug );
		$this->assertEquals( 'tag3', $tags[2]->slug );

		$post = $posts[5];
		$this->assertEquals( '1-col page', $post->post_title );
		$this->assertEquals( '1-col-page', $post->post_name );
		$this->assertEquals( $admin->ID, $post->post_author );
		$this->assertEquals( 'page', $post->post_type );
		$this->assertEquals( 'publish', $post->post_status );
		$this->assertEquals( 0, $post->post_parent );
		$this->assertEquals( 'onecolumn-page.php',
				get_post_meta( $post->ID, '_wp_page_template', true ) );

		$post = $posts[6];
		$this->assertEquals( 'Draft Page', $post->post_title );
		$this->assertEquals( '', $post->post_name );
		$this->assertEquals( $admin->ID, $post->post_author );
		$this->assertEquals( 'page', $post->post_type );
		$this->assertEquals( 'draft', $post->post_status );
		$this->assertEquals( 0, $post->post_parent );
		$this->assertEquals( 'default',
				get_post_meta( $post->ID, '_wp_page_template', true ) );

		$post = $posts[7];
		$this->assertEquals( 'Parent Page', $post->post_title );
		$this->assertEquals( 'parent-page', $post->post_name );
		$this->assertEquals( $admin->ID, $post->post_author );
		$this->assertEquals( 'page', $post->post_type );
		$this->assertEquals( 'publish', $post->post_status );
		$this->assertEquals( 0, $post->post_parent );
		$this->assertEquals( 'default',
				get_post_meta( $post->ID, '_wp_page_template', true ) );

		$post = $posts[8];
		$this->assertEquals( 'Child Page', $post->post_title );
		$this->assertEquals( 'child-page', $post->post_name );
		$this->assertEquals( $admin->ID, $post->post_author );
		$this->assertEquals( 'page', $post->post_type );
		$this->assertEquals( 'publish', $post->post_status );
		$this->assertEquals( $posts[7]->ID, $post->post_parent );
		$this->assertEquals( 'default',
				get_post_meta( $post->ID, '_wp_page_template', true ) );

		$post = $posts[9];
		$this->assertEquals( 'Sample Page', $post->post_title );
		$this->assertEquals( 'sample-page', $post->post_name );
		$this->assertEquals( $admin->ID, $post->post_author );
		$this->assertEquals( 'page', $post->post_type );
		$this->assertEquals( 'publish', $post->post_status );
		$this->assertEquals( 0, $post->post_parent );
		$this->assertEquals( 'default',
				get_post_meta( $post->ID, '_wp_page_template', true ) );

		$post = $posts[10];
		$this->assertEquals( 'Hello world!', $post->post_title );
		$this->assertEquals( 'hello-world', $post->post_name );
		$this->assertEquals( $author->ID, $post->post_author );
		$this->assertEquals( 'post', $post->post_type );
		$this->assertEquals( 'publish', $post->post_status );
		$this->assertEquals( 0, $post->post_parent );
		$cats = wp_get_post_categories( $post->ID );
		$this->assertEquals( 1, count( $cats ) );
	}

	/**
	 * Uses Reflection to call otherwise inaccessible methods.
	 *
	 * Although typically this wouldn't be used, the importer being tested uses
	 * a number of steps in protected methods.
	 *
	 * @param object $object
	 * @param string $name
	 * @param array $args
	 *        	optional
	 */
	protected static function call( $object, $name, array $args = array() ) {
		$class = new \ReflectionClass( $object );
		$method = $class->getMethod( $name );
		$method->setAccessible( true );
		return $method->invokeArgs( $object, $args );
	}

	/**
	 * Uses Reflection to set otherwise inaccessible properties.
	 *
	 * @param object $object
	 * @param string $name
	 * @param mixed $value
	 */
	protected static function set( $object, $name, $value ) {
		$class = new \ReflectionClass( $object );
		$variable = $class->getProperty( $name );
		$variable->setAccessible( true );
		return $variable->setValue( $object, $value );
	}

	/**
	 * Uses Reflection to get otherwise inaccessible properties.
	 *
	 * @param object $object
	 * @param string $name
	 * @throws \ReflectionException
	 */
	protected static function get( $object, $name ) {
		$class = new \ReflectionClass( $object );
		$variable = $class->getProperty( $name );
		$variable->setAccessible( true );
		return $variable->getValue( $object );
	}
}

