<?php
/*
Plugin Name: WordPress Importer
Plugin URI: http://wordpress.org/extend/plugins/wordpress-importer/
Description: Import posts, pages, comments, custom fields, categories, tags and more from a WordPress export file.
Author: wordpressdotorg
Author URI: http://wordpress.org/
Version: 1.0.0-beta
Text Domain: wordpress-importer
License: GPL version 2 or later - http://www.gnu.org/licenses/old-licenses/gpl-2.0.html

Forked from the WordPress Importer (http://wordpress.org/extend/plugins/wordpress-importer/)
at revision 718670.
*/

if ( ! defined( 'WP_LOAD_IMPORTERS' ) && ! defined( 'DOING_CRON' ) )
	return;

/** Display verbose errors */
if ( ! defined( 'IMPORT_DEBUG' ) )
	define( 'IMPORT_DEBUG', false );

// Load importer API, cron class, and WXR parsers
require_once ABSPATH . 'wp-admin/includes/import.php';
require dirname(__FILE__) . '/class-wp-importer-cron.php';
require dirname( __FILE__ ) . '/class-wxr-parser.php';

// Load specific cron importer implementation
if ( ! class_exists( 'WordPress_Importer' ) ) {
    require dirname(__FILE__) . '/class-wordpress-importer.php';
}

/**
 * WordPress Importer class for managing the import UI for a WXR file
 *
 * @package WordPress
 * @subpackage Importer
 */
class WP_Import extends WP_Importer {

	var $id; // WXR attachment ID

	/**
	 * Instance of the cron importer.
	 *
	 * @var WordPress_Importer
	 */
	protected $importer;

	function __construct() {
	    $this->importer = new WordPress_Importer();
	}

	/**
	 * Registered callback function for the WordPress Importer
	 *
	 * Manages the three separate stages of the WXR import process
	 */
	function dispatch() {
		$this->header();

		$step = empty( $_GET['step'] ) ? 0 : (int) $_GET['step'];
		switch ( $step ) {
			case 0:
				$this->greet();
				break;
			case 1:
				check_admin_referer( 'import-upload' );
				if ( $this->handle_upload() )
					$this->import_options();
				break;
			case 2:
				check_admin_referer( 'import-wordpress' );
				$this->id = (int) $_POST['import_id'];
				$file = get_attached_file( $this->id );
				set_time_limit(0);
				$this->import( $file );
				break;
			case 3:
				// status
				check_admin_referer( 'import-wordpress' );
				$this->status();
				break;
		}

		$this->footer();
	}

	/**
	 * The main controller for the actual import stage.
	 *
	 * @param string $file Path to the WXR file for importing
	 */
	function import( $file ) {
		add_filter( 'import_post_meta_key', array( $this, 'is_valid_meta_key' ) );
		add_filter( 'http_request_timeout', array( &$this, 'bump_request_timeout' ) );

		$this->import_start( $file );

		if ( wp_next_scheduled( 'wp_cron_importer_hook', array(
				$file
		) ) !== false ) {
			// cron job is not finished; can't end import yet
			$this->status();
		} else
			$this->import_end();
	}

	/**
	 * Parses the WXR file and prepares us for the task of processing parsed data
	 *
	 * @param string $file Path to the WXR file for importing
	 */
	function import_start( $file ) {
		if ( ! is_file($file) ) {
			echo '<p><strong>' . __( 'Sorry, there has been an error.', 'wordpress-importer' ) . '</strong><br />';
			echo __( 'The file does not exist, please try again.', 'wordpress-importer' ) . '</p>';
			$this->footer();
			die();
		}

		$import_data = $this->importer->do_parse( $file );

		if ( is_wp_error( $import_data ) ) {
			echo '<p><strong>' . __( 'Sorry, there has been an error.', 'wordpress-importer' ) . '</strong><br />';
			echo esc_html( $import_data->get_error_message() ) . '</p>';
			$this->footer();
			die();
		}

		$this->importer->set_param( 'fetch_attachments',
		    (! empty( $_POST['fetch_attachments'] ) &&
		        $this->importer->is_allowed_attachment_fetch()) );
		$this->importer->set_param( 'user_new',
		    (! empty( $_POST['user_new'] ) ? $_POST['user_new'] : array()) );
		$this->importer->set_param( 'user_map',
		    (! empty( $_POST['user_map'] ) ? $_POST['user_map'] : array()) );
		$this->importer->set_param( 'imported_authors',
		    (! empty( $_POST['imported_authors'] ) ? $_POST['imported_authors'] : array()) );

		$this->importer->advance('start');

		$this->importer->schedule_import_job( 'do_import', array( $file ) );
		$this->importer->save_vars();

		do_action( 'import_start' );
	}

	/**
	 * Performs post-import cleanup of files and the cache
	 */
	function import_end() {
		// delete the WXR attachment
		wp_import_cleanup( $this->id );

		// run the cron importer's cleanup routine
		$this->importer->do_cleanup();

		do_action( 'import_end' );
	}

	/**
	 * Handles the WXR upload and initial parsing of the file to prepare for
	 * displaying author import options
	 *
	 * @return bool False if error uploading or invalid file, true otherwise
	 */
	function handle_upload() {
		$file = wp_import_handle_upload();
		if ( isset( $file['error'] ) ) {
			echo '<p><strong>' . __( 'Sorry, there has been an error.',
					'wordpress-importer' ) . '</strong><br />';
			echo esc_html( $file['error'] ) . '</p>';
			return false;
		} else
			if ( ! file_exists( $file['file'] ) ) {
				echo '<p><strong>' . __( 'Sorry, there has been an error.',
						'wordpress-importer' ) . '</strong><br />';
				printf(
						__(
								'The export file could not be found at <code>%s</code>. It is likely that this was caused by a permissions problem.',
								'wordpress-importer' ),
						esc_html( $file['file'] ) );
				echo '</p>';
				return false;
			}

		$this->id = (int) $file['id'];
		// do a dry run of the parse just to get author data
		$import_data = $this->importer->do_parse( $file['file'], true );

		if ( is_wp_error( $import_data ) ) {
			/* @var $import_data WP_Error */
			if ( 'import_wxr_version_error' == $import_data->get_error_code() ) {
				echo '<div class="error"><p><strong>';
				printf(
						__(
								'This WXR file (version %s) may not be supported by this version of the importer. Please consider updating.',
								'wordpress-importer' ),
						esc_html( $import_data->get_error_data() ) );
				echo '</strong></p></div>';
			}
			echo '<p><strong>' . __( 'Sorry, there has been an error.',
					'wordpress-importer' ) . '</strong><br />';
			echo esc_html( $import_data->get_error_message() ) . '</p>';
			return false;
		}

		return true;
	}

	/**
	 * Display pre-import options, author importing/mapping and option to
	 * fetch attachments
	 */
	function import_options() {
		$authors = $this->importer->get_authors();
		$j = 0;

		if(is_wp_error($authors)) {
			echo '<p><strong>' . __( 'Sorry, there has been an error.',
			    'wordpress-importer' ) . '</strong><br />';
			echo esc_html( $authors->get_error_message() ) . '</p>';
		}
?>
<form
	action="<?php echo admin_url( 'admin.php?import=wordpress&amp;step=2' ); ?>"
	method="post">
	<?php wp_nonce_field( 'import-wordpress' ); ?>
	<input type="hidden" name="import_id" value="<?php echo $this->id; ?>" />

<?php if ( ! empty( $authors ) && !is_wp_error($authors) ) : ?>
	<h3><?php _e( 'Assign Authors', 'wordpress-importer' ); ?></h3>
	<p><?php _e( 'To make it easier for you to edit and save the imported content, you may want to reassign the author of the imported item to an existing user of this site. For example, you may want to import all the entries as <code>admin</code>s entries.', 'wordpress-importer' ); ?></p>
<?php if ( $this->importer->is_allowed_user_creation() ) : ?>
	<p><?php printf( __( 'If a new user is created by WordPress, a new password will be randomly generated and the new user&#8217;s role will be set as %s. Manually changing the new user&#8217;s details will be necessary.', 'wordpress-importer' ), esc_html( get_option('default_role') ) ); ?></p>
<?php endif; ?>
	<ol id="authors">
<?php foreach ( $authors as $author ) : ?>
		<li><?php $this->author_select( $j++, $author ); ?></li>
<?php endforeach; ?>
	</ol>
<?php endif; ?>

<?php if ( $this->importer->is_allowed_attachment_fetch() ) : ?>
	<h3><?php _e( 'Import Attachments', 'wordpress-importer' ); ?></h3>
	<p>
		<input type="checkbox" value="1" name="fetch_attachments"
			id="import-attachments" /> <label for="import-attachments"><?php _e( 'Download and import file attachments', 'wordpress-importer' ); ?></label>
	</p>
<?php endif; ?>

	<p class="submit">
		<input type="submit" class="button"
			value="<?php esc_attr_e( 'Submit', 'wordpress-importer' ); ?>" />
	</p>
</form>
<?php
	}

	/**
	 * Display import options for an individual author. That is, either create
	 * a new user based on import info or map to an existing user
	 *
	 * @param int $n Index for each author in the form
	 * @param array $author Author information, e.g. login, display name, email
	 */
	function author_select( $n, $author ) {
		$version = $this->importer->get_version();
		$create_users = $this->importer->is_allowed_user_creation();

		_e( 'Import author:', 'wordpress-importer' );
		echo ' <strong>' . esc_html( $author['author_display_name'] );
		if ( $version != 1.0 ) echo ' (' . esc_html( $author['author_login'] ) . ')';
		echo '</strong><br />';

		if ( $version != 1.0 )
			echo '<div style="margin-left:18px">';

		if ( $create_users ) {
			if ( $version != 1.0 ) {
				_e( 'or create new user with login name:', 'wordpress-importer' );
				$value = '';
			} else {
				_e( 'as a new user:', 'wordpress-importer' );
				$value = esc_attr( sanitize_user( $author['author_login'], true ) );
			}

			echo ' <input type="text" name="user_new['.$n.']" value="'. $value .'" /><br />';
		}

		if ( ! $create_users && $version == 1.0 )
			_e( 'assign posts to an existing user:', 'wordpress-importer' );
		else
			_e( 'or assign posts to an existing user:', 'wordpress-importer' );
		wp_dropdown_users( array( 'name' => "user_map[$n]", 'multi' => true, 'show_option_all' => __( '- Select -', 'wordpress-importer' ) ) );
		echo '<input type="hidden" name="imported_authors['.$n.']" value="' . esc_attr( $author['author_login'] ) . '" />';

		if ( $version != 1.0 )
			echo '</div>';
	}

	// Display import page title
	function header() {
		echo '<div class="wrap">';
		screen_icon();
		echo '<h2>' . __( 'Import WordPress', 'wordpress-importer' ) . '</h2>';

		$updates = get_plugin_updates();
		$basename = plugin_basename(__FILE__);
		if ( isset( $updates[$basename] ) ) {
			$update = $updates[$basename];
			echo '<div class="error"><p><strong>';
			printf( __( 'A new version of this importer is available. Please update to version %s to ensure compatibility with newer export files.', 'wordpress-importer' ), $update->update->new_version );
			echo '</strong></p></div>';
		}
	}

	// Close div.wrap
	function footer() {
		echo '</div>';
	}

	/**
	 * Display introductory text and file upload form
	 */
	function greet() {
		echo '<div class="narrow">';
		echo '<p>'.__( 'Howdy! Upload your WordPress eXtended RSS (WXR) file and we&#8217;ll import the posts, pages, comments, custom fields, categories, and tags into this site.', 'wordpress-importer' ).'</p>';
		echo '<p>'.__( 'Choose a WXR (.xml) file to upload, then click Upload file and import.', 'wordpress-importer' ).'</p>';
		wp_import_upload_form( 'admin.php?import=wordpress&amp;step=1' );
		echo '</div>';
	}

	function status() {
		// since the instance in this class should have loaded its variables
		// from the options, we SHOULD be able to query it for status
		echo '<h3>Import Status</h3>';
		if ( ! is_wp_error( $this->importer->get_step() )) :
			if ( 'finish' != $this->importer->get_step() ) {
				echo '<p>' . __(
						'The importer is now processing the data you have selected for import. It will execute tasks in the background; you can monitor the status by the refresh button below. The process may take a few minutes.',
						'wordpress-importer' ) . '</p>';
			    $this->status_buttons();
			}
			printf(
					'<p><strong>' .
							 __( 'Current step: %s', 'wordpress-importer' ) .
							 '</strong></p>', $this->importer->get_step() );
?>
<dl>
	<dt>Authors imported and mapped:</dt>
	<dd><?php printf( '%d', $this->importer->count( 'processed_author_mappings' ) ); ?></dd>
	<dt>Categories, tags, and terms imported:</dt>
	<dd><?php printf( '%d', $this->importer->count( 'processed_terms' ) ); ?></dd>
	<dt>Posts, attachments, menu items, and custom post types imported:</dt>
	<dd><?php printf( '%d of %d', $this->importer->count( 'processed_posts' ) + $this->importer->count( 'processed_menu_items' ),
		$this->importer->count( 'posts' ) ); ?></dd>
<?php if( 'finish' != $this->importer->get_step() && ! $this->importer->get_error() ) : ?>
	<dt>Next cron run:</dt>
	<dd><?php echo date_i18n( get_option( 'date_format' ) . ' ' . get_option(
		'time_format' ),
		wp_next_scheduled( 'wp_cron_importer_hook',
				array(
						$this->importer->get_param( 'wxr_file' )
				) ) ); ?></dd>
<?php endif; ?>
</dl>

<?php
		endif;

		$error = $this->importer->get_error();
		$warnings = $this->importer->get_warnings();
		if(! empty( $warnings ) ) {
			echo '<h3>';
			printf(
					__( 'The import process resulted in %d warnings',
							'wordpress-importer' ), count( $warnings ) );
			echo '</h3><div class="error">';
			foreach ( $warnings as $warning ) {
				/* @var $warning WP_Error */
				echo '<p>' . $warning->get_error_message() . '</p>';
			}
			echo '</div>';
		}

		if ( IMPORT_DEBUG ) {
			echo '<h3>';
			echo __( 'Debug Data', 'wordpress-importer' );
			echo '</h3>';
			echo '<pre>';
			echo _wp_specialchars(
					print_r( get_option( get_class( $this->importer ) ),
							true ) );
			echo '</pre>';
		}

		// if done, run the import_end() cleanup; note that this will wipe
		// any error and warning data, so we need to get all that first (above)
		if ( is_wp_error( $error ) || $_POST['abort'] )
			$this->importer->advance( 'finish' );

		if ( 'finish' == $this->importer->get_step() ) {
			echo '<h3>';
			echo __( 'All done.', 'wordpress-importer' );
			echo '</h3><p>';
			if ( count( $this->importer->get_param( 'user_new', array() ) ) > 0 ) {
				echo __(
						'Remember to update the passwords and roles of imported users.',
						'wordpress-importer' );
				echo ' ';
			}
			echo __( 'Have fun!', 'wordpress-importer' );
			echo '</p>';

			$this->import_end();
		} else {
			$this->status_buttons();
		} // end else
	}

	/**
	 * Added to http_request_timeout filter to force timeout at 60 seconds during import
	 * @return int 60
	 */
	function bump_request_timeout( $val ) {
		return 60;
	}

	function status_buttons () {
?>
<form
	action="<?php echo admin_url( 'admin.php?import=wordpress&amp;step=3' ); ?>"
	method="post">
	<?php wp_nonce_field( 'import-wordpress' ); ?>
	<p class="submit">
		<input type="submit" class="button"
			value="<?php esc_attr_e( 'Refresh', 'wordpress-importer' ); ?>" /> <input
			type="submit" class="button" name="abort"
			value="<?php esc_attr_e( 'Abort', 'wordpress-importer' ); ?>" />
	</p>
</form>
<?php
	}
}

function wordpress_importer_init() {
	load_plugin_textdomain( 'wordpress-importer', false,
			dirname( plugin_basename( __FILE__ ) ) . '/languages' );

	/**
	 * WordPress Importer object for registering the import callback
	 *
	 * @global WP_Import $wp_import
	 */
	$GLOBALS['wp_import'] = new WP_Import();
	register_importer( 'wordpress', 'WordPress',
			__(
					'Import <strong>posts, pages, comments, custom fields, categories, and tags</strong> from a WordPress export file.',
					'wordpress-importer' ),
			array(
					$GLOBALS['wp_import'],
					'dispatch' ) );
}
add_action( 'init', 'wordpress_importer_init' );
